/*-------------------------------------------------------------------------+
| Copyright 2020 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.view.fx;

import static org.fortiss.tooling.base.ui.annotation.view.fx.AnnotationsFXUtils.SELECTED_ANNOTATION_BACKGROUND_COLOR;
import static org.fortiss.tooling.base.ui.annotation.view.fx.AnnotationsFXUtils.getBackgroundColorForEntry;

import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.annotation.valueprovider.IAnnotationValueProvider;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.common.ui.javafx.control.treetableview.DynamicTreeTableUIProviderBase;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;

import javafx.scene.paint.Color;

/**
 * UIProvider for the Annotation TreeTable.
 */
/* package */ final class AnnotationTreeTableUIProvider
		extends DynamicTreeTableUIProviderBase<AnnotationEntry> {

	/** The {@link AnnotationViewFXController} this UI provider was created for. */
	private final AnnotationViewFXController viewController;

	/**
	 * Constructor.
	 */
	AnnotationTreeTableUIProvider(AnnotationViewFXController annotationViewFXController) {
		this.viewController = annotationViewFXController;
	}

	/** {@inheritDoc} */
	@Override
	public String getLabel(AnnotationEntry entry, int column) {
		IModelElement element = entry.getModelElement();

		switch(column) {
			case 0:
				return this.viewController.getNameForAnnotationEntry(entry);
			case 1:
				if(element instanceof INamedCommentedElement) {
					return ((INamedCommentedElement)element).getComment();
				}
				return "";
			default:
				IAnnotatedSpecification columnSpec = this.viewController.colIdxAnnotationMap
						.get(column).getAnnotatedSpecification();

				Object specificationValue = entry.getSpecificationValue(columnSpec.getClass());
				if(specificationValue != null) {
					return specificationValue.toString();
				}

				return "";
		}
	}

	/** {@inheritDoc} */
	@Override
	public Color getBackgroundColor(int column, AnnotationEntry element) {
		if(element.getModelElement().equals(this.viewController.selected)) {
			return SELECTED_ANNOTATION_BACKGROUND_COLOR;
		}

		if(column > 1 && this.viewController.colIdxAnnotationMap.containsKey(column)) {
			Class<IAnnotatedSpecification> specType = this.viewController.colIdxAnnotationMap
					.get(column).getAnnotatedSpecificationType();
			return getBackgroundColorForEntry(element, specType);
		}

		return super.getBackgroundColor(column, element);
	}

	/** {@inheritDoc} */
	@Override
	public void updateValue(AnnotationEntry element, int column, Object value) {
		IAnnotatedSpecification specification =
				this.viewController.colIdxAnnotationMap.get(column).getAnnotatedSpecification();
		this.viewController.setAnnotationEntryValue(specification, element, (String)value);

		super.updateValue(element, column, value);
	}

	/** {@inheritDoc} */
	@Override
	public boolean isEditable(int column, AnnotationEntry element) {
		if(column > 1 && this.viewController.colIdxAnnotationMap.containsKey(column)) {
			IAnnotatedSpecification spec =
					this.viewController.colIdxAnnotationMap.get(column).getAnnotatedSpecification();
			IAnnotationValueProvider<IAnnotatedSpecification> valueProvider =
					element.getAnnotationValueProvider(spec.getClass());
			return valueProvider != null && valueProvider.canEdit(spec);
		}

		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isEditable(int column) {

		// The first two columns are not supposed to be editable.
		if(column <= 1) {
			return false;
		}

		for(AnnotationEntry ae : this.viewController.root.elements) {
			// If any cell is editable in this column, the column is editable.
			if(isEditable(column, ae)) {
				return true;
			}
		}

		return false;
	}
}
